#!/usr/bin/env python
import PySimpleGUI as sg
from PIL import Image
import cv2
import io
import imageio
def loadVideo():
	filename = sg.popup_get_file('Filename to gif')
	return filename
def main():
	
	# ---===--- Get the filename --- #
	skipRate=3
	gifFps=10
	videoFile=loadVideo()
	# scale=0.5
	try:
		outFile=videoFile[:-3] + 'gif'
		cap = cv2.VideoCapture(videoFile)
		videoWidth=cap.get(3)
		videoHeight=cap.get(4)
		videoFps=cap.get(5)
		frameTotal = cap.get(7)
	except:
		print("cannot play")
		return

	sg.theme('Black')

	# ---===--- define the window layout --- #
	layout = [[sg.Text('OpenCV Demo', size=(40, 1), font='Helvetica 20',key='-text-')],
			  [sg.Image(filename='', key='-image-')],

			  [sg.Text('Video Sample Rate'),sg.Slider(range=(skipRate, 300),
						size=(60, 10), orientation='h', key='-slider-')],
			  [sg.Text('Output Gif FPS'),sg.Slider(range=(10, 60),
						size=(60, 10), orientation='h', key='-slider2-')],
			  [sg.Text('Video Resize Rate'),sg.Slider(range=(2, 10),
						size=(60, 10),orientation='h', key='-slider3-')],
			  [sg.Button('Start', size=(7, 1), pad=((600, 0), 3), font='Helvetica 14')],
			  [sg.Button('Restart', size=(7, 1), pad=((600, 0), 3), font='Helvetica 14')],
			  [sg.Button('Exit', size=(7, 1), pad=((600, 0), 3), font='Helvetica 14')]]

	# create the window and show it without the plot
	window = sg.Window('Demo Application - video2gif', layout, no_titlebar=False, location=(0, 0))

	# locate the elements we'll be updating. Does the search only 1 time
	image_elem = window['-image-']
	text_elem = window['-text-']
	slider_elem = window['-slider-']
	gifSliderElem = window['-slider2-']
	sizeSliderElem = window['-slider3-']
	
	# ---===--- LOOP through video file by frame --- #
	cur_frame = 0
	startProcess=False
	ret, firstFrame = cap.read()

	while True:
		event, values = window.read(timeout=0)
		if event in ('Start', None):
			startProcess=True
		if event in ('Exit', None):
			break
		if(startProcess==False):
			skipRate = int(values['-slider-'])
			gifFps = int(values['-slider2-'])
			scale = int(values['-slider3-'])
			slider_elem.update(skipRate)
			gifSliderElem.update(gifFps)
			sizeSliderElem.update(scale)
			firstFrameDisplay=cv2.resize(firstFrame,(0,0),fx=1/scale,fy=1/scale)
			framerCounter=0
			cap.set(cv2.CAP_PROP_POS_FRAMES, framerCounter)
			gifEstLenght="EST out put GIF length:"+ str(round((frameTotal/skipRate)/gifFps,2))+" s"
			text_elem.update(gifEstLenght)
			imgbytes = cv2.imencode('.png', firstFrameDisplay)[1].tobytes()  # ditto
			image_elem.update(data=imgbytes)
		else:
			eta="strating...."
			text_elem.update(eta)
			with imageio.get_writer(outFile, duration=1/gifFps, mode='I') as writer:

				while True:
					event, values = window.read(timeout=0)
					if event in ('Restart', None):
						break
					cap.set(cv2.CAP_PROP_POS_FRAMES, framerCounter)
					ret, frame = cap.read()
					if not ret:  # if out of data stop looping
						break
					framerCounter+=skipRate
					frame=cv2.resize(frame,(0,0),fx=1/scale,fy=1/scale)		
					imgbytes = cv2.imencode('.png', frame)[1].tobytes()  # ditto
					image_elem.update(data=imgbytes)
					frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
					writer.append_data(frame)
					eta="ETA-"+str(round((framerCounter/frameTotal)*100,2))+"%"
					text_elem.update(eta)
					slider_elem.update(skipRate)
					gifSliderElem.update(gifFps)
					if cv2.waitKey(1) == ord('q'):
						cap.set(cv2.CAP_PROP_POS_FRAMES, 0)
						startProcess=False
						break
			# 
			startProcess=False
	cap.release()













	
main()


#!/usr/bin/env python
# ou zheng
# 2020/1.2
import os

import cv2
import imageio
import click

skipRate = 30


@click.command(name='A script that converts video to GIF.', help='Converts video to GIF.')
@click.option('-i', '--input', type=click.Path(exists=True), default='../video/demo.mov', help='input video file',
              prompt='Enter path of the video file to convert to GIF.')
@click.option('-o', '--output', help='output gif file path and name.')
@click.option('-s', '--scale', type=click.INT, default=1, help="Scale the factor for frame_x and frame_y.")
@click.option('-r', '--resolution', help="The resolution factor for factor ( For Compress ), like 1920x1080.",
              default="0x0")
@click.option('-fps', '--gif-fps', type=click.INT, default=10, help="The fps of the GIF.",
              prompt="Enter the fps of the GIF.")
def main(input: str, output: str, scale: int, resolution: str, gif_fps: int):
    videoName = input
    if output:
        outName = output
    else:
        out_dir = os.path.dirname(videoName)
        outName = os.path.join(out_dir, videoName[:-3] + 'gif')
    print("out:", outName)
    d_size = (0, 0)
    if resolution:
        r_x, r_y = resolution.split("x")
        d_size = (int(r_x), int(r_y))

    print("resolution:", d_size)
    cap = cv2.VideoCapture(videoName)
    videoWidth = cap.get(3)
    videoHeight = cap.get(4)
    videoFps = cap.get(5)
    print("video original fps:", videoFps)
    frameTotal = cap.get(7)
    frameForOutput = frameTotal / skipRate

    with imageio.get_writer(outName, duration=1 / gif_fps, mode='I') as writer:
        framerCounter = 0
        while True:
            ret, frame = cap.read()
            if not ret:
                print("finished")
                break
            framerCounter += 1
            if framerCounter % 10 == 0:
                frame = cv2.resize(frame, d_size, fx=scale, fy=scale)
                cv2.imshow('frame', frame)
                frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
                writer.append_data(frame)
                print("ETA-", round((framerCounter / frameTotal) * 100, 2), "%")
            else:
                continue

            if cv2.waitKey(1) == ord('q'):
                break
    cap.release()


if __name__ == '__main__':
    main()



"""
The build/compilations setup
>> pip install -r requirements.txt
>> python setup.py install
"""
import pip
import logging
import pkg_resources

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def _parse_requirements(file_path):
    pip_ver = pkg_resources.get_distribution('pip').version
    pip_version = list(map(int, pip_ver.split('.')[:2]))
    if pip_version >= [6, 0]:
        raw = pip.req.parse_requirements(file_path,
                                         session=pip.download.PipSession())
    else:
        raw = pip.req.parse_requirements(file_path)
    return [str(i.req) for i in raw]


# parse_requirements() returns generator of pip.req.InstallRequirement objects
try:
    install_reqs = _parse_requirements("requirements.txt")
except Exception:
    logging.warning('Fail load requirements file, so using default ones.')
    install_reqs = []

long_description = open('README.rst').read()

setup(
    name='vgif',
    version='1.0.5',
    url='https://github.com/Haoke98/video2gif',
    author='Haoke98',
    author_email='Haoke98@outlook.com',
    license='MIT',
    description='https://github.com/Haoke98/video2gif',
    packages=["vgif"],
    install_requires=install_reqs,
    include_package_data=True,
    python_requires='>=3.7',
    long_description=long_description,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Visualization",
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords="image video gif",
    entry_points={
        'console_scripts': [
            'vgif-gui=vgif.main:main',
            'vgif=vgif.video2gif:main',
        ],
    },
)
